@@ -1,7 +1,7 @@
 /*
  * lmms_math.h - defines math functions
  *
- * Copyright (c) 2004-2006 Tobias Doerffel <tobydox/at/users.sourceforge.net>
+ * Copyright (c) 2004-2008 Tobias Doerffel <tobydox/at/users.sourceforge.net>
  * 
  * This file is part of Linux MultiMedia Studio - http://lmms.sourceforge.net
  *
@@ -27,15 +27,72 @@
 #define _LMMS_MATH_H
 
 
+#ifdef __INTEL_COMPILER
 
+#include <math.h>
+
+static inline float absFraction( const float _x )
+{
+	return( _x - ( _x >= 0.0f ? floorf( _x ) : floorf( _x ) - 1 ) );
+}
 
-// Equivalent to _x - floorf( _x )
 static inline float fraction( const float _x )
+{
+	return( _x - floorf( _x ) );
+}
+
+#else
+
+static inline float absFraction( const float _x )
 {
 	return( _x - ( _x >= 0.0f ? static_cast<int>( _x ) :
 						static_cast<int>( _x ) - 1 ) );
 }
 
+static inline float fraction( const float _x )
+{
+	return( _x - static_cast<int>( _x ) );
+}
+
+
+#if 0
+// SSE3-version
+static inline float absFraction( float _x )
+{
+	unsigned int tmp;
+	asm(
+		"fld %%st\n\t"
+		"fisttp %1\n\t"
+		"fild %1\n\t"
+		"ftst\n\t"
+		"sahf\n\t"
+		"jae 1f\n\t"
+		"fld1\n\t"
+		"fsubrp %%st, %%st(1)\n\t"
+	"1:\n\t"
+		"fsubrp %%st, %%st(1)"
+		: "+t"( _x ), "=m"( tmp )
+		:
+		: "st(1)", "cc" );
+	return( _x );
+}
+
+static inline float absFraction( float _x )
+{
+	unsigned int tmp;
+	asm(
+		"fld %%st\n\t"
+		"fisttp %1\n\t"
+		"fild %1\n\t"
+		"fsubrp %%st, %%st(1)"
+		: "+t"( _x ), "=m"( tmp )
+		:
+		: "st(1)" );
+	return( _x );
+}
+#endif
+
+#endif
 
 
 
